from enum import Enum, auto

# Definícia typov tokenov rozpoznávaných lexerom
class TokenType(Enum):
    NUMBER = auto()    # Číselná hodnota (prirodzené číslo)
    PLUS = auto()      # Operátor sčítania '+'
    MINUS = auto()     # Operátor odčítania '-'
    MULTIPLY = auto()  # Operátor násobenia '*'
    POWER = auto()     # Operátor umocnenia '^'
    LPAREN = auto()    # Ľavá zátvorka '('
    RPAREN = auto()    # Pravá zátvorka ')'
    EOF = auto()       # Koniec vstupu (End Of File)

# Trieda reprezentujúca token - kombinácia typu a atribútu (hodnoty)
class Token:
    def __init__(self, type, value=None):
        self.type = type    # Typ tokenu (z enum TokenType)
        self.attribute = value  # Hodnota tokenu (relevantná hlavne pre čísla)
    
    def __repr__(self):
        # Reťazcová reprezentácia tokenu
        if self.attribute is not None:
            return f"Token({self.type}, {self.attribute})"
        return f"Token({self.type})"

# Lexikálny analyzátor - rozpoznáva a extrahuje tokeny zo vstupného reťazca
class Lexer:
    def __init__(self, input_text):
        self.input_text = input_text                       # Vstupný analýzovaný vstupný text
        self.pos = 0                                  # Aktuálne analyzovaná pozícia vo vstupnom texte
        self.current_char = self.input_text[0] if input_text else None # Aktuálny znak alebo None ak je vstupný text prázdny
    
    def advance(self):
        """Posunie pozíciu na ďalší znak a aktualizuje current_char.
        Ak sa dostane za koniec textu, nastaví current_char na None."""
        self.pos += 1
        if self.pos < len(self.input_text):
            self.current_char = self.input_text[self.pos]
        else:
            self.current_char = None  # Dosiahli sme koniec textu
    
    def skip_whitespace(self):
        """Preskočí všetky medzery a nové riadky vo vstupe."""
        while self.current_char is not None and self.current_char.isspace():
            self.advance()
    
    def number(self):
        """Vracia token typu NUMBER s číselnou hodnotou."""
        # TODO: Implementujte rozpoznávanie čísla pomocou Hornerovej schémy


    
    def get_next_token(self):
        """Hlavná metóda lexikálneho analyzátora.
        Analyzuje text a vracia ďalší token v poradí."""
        while self.current_char is not None:
            # Ignorovanie medzier a formátovanie
            if self.current_char.isspace():
                self.skip_whitespace()
                continue
            
            # Rozpoznávanie čísla 
            if self.current_char.isdigit():
                return self.number()
            
            # Rozpoznávanie operátorov a zátvoriek - jeden znak = jeden token
            if self.current_char == '+':
                self.advance()
                return Token(TokenType.PLUS)
            
            if self.current_char == '-':
                self.advance()
                return Token(TokenType.MINUS)
            
            if self.current_char == '*':
                self.advance()
                return Token(TokenType.MULTIPLY)
            
            if self.current_char == '^':
                self.advance()
                return Token(TokenType.POWER)
            
            if self.current_char == '(':
                self.advance()
                return Token(TokenType.LPAREN)
            
            if self.current_char == ')':
                self.advance()
                return Token(TokenType.RPAREN)
            
            # Ak narazíme na neznámy znak, vyhodíme chybu
            raise ValueError(f"Nerozpoznaný znak: '{self.current_char}'")
        
        # Keď prejdeme celý text, vrátime token konca vstupu
        return Token(TokenType.EOF)